package org.netkernel.client.http.endpoint;

import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class DefaultConnectionManagerAccessor extends StandardAccessorImpl
{
	public static final int MAXIMUM_TOTAL_HTTP_CLIENT_CONNECTIONS=64;
	public static final int MAXIMUM_TOTAL_HTTP_CLIENT_CONNECTIONS_PER_HOST=8;
	public static final String MAXIMUM_TOTAL_HTTP_CLIENT_CONNECTIONS_PER_HOST_PROPERTY="http.client.max.total.connections.per.host";
	public static final String MAXIMUM_TOTAL_HTTP_CLIENT_CONNECTIONS_PROPERTY="http.client.max.total.connections";
	
	public DefaultConnectionManagerAccessor()
	{	this.declareThreadSafe();		
	}

	@Override
	public void onSource(INKFRequestContext aContext) throws Exception
	{	SchemeRegistry sreg=new SchemeRegistry();
		sreg.register(new Scheme("http", 80, PlainSocketFactory.getSocketFactory()));
		SSLSocketFactory sf=SSLSocketFactory.getSocketFactory();
		Scheme https=new Scheme("https", 443, sf);
		sreg.register(https);
		ThreadSafeClientConnManager m = new ThreadSafeClientConnManager(sreg);
		int connsPerHost, totalConns;
		try
		{ totalConns=aContext.getKernelContext().getKernel().getConfiguration().getInt(MAXIMUM_TOTAL_HTTP_CLIENT_CONNECTIONS_PROPERTY);
		}
		catch(Exception e)
		{	totalConns=MAXIMUM_TOTAL_HTTP_CLIENT_CONNECTIONS;		
		}
		try
		{ connsPerHost=aContext.getKernelContext().getKernel().getConfiguration().getInt(MAXIMUM_TOTAL_HTTP_CLIENT_CONNECTIONS_PER_HOST_PROPERTY);
		}
		catch(Exception e)
		{	connsPerHost=MAXIMUM_TOTAL_HTTP_CLIENT_CONNECTIONS_PER_HOST;			
		}
		m.setMaxTotal(totalConns);
		m.setDefaultMaxPerRoute(connsPerHost);
		aContext.createResponseFrom(m);
	}
}
