package org.netkernel.client.http.endpoint;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.commonshttp.CommonsHttpOAuthConsumer;

import org.apache.commons.httpclient.methods.HttpPatch;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.NTCredentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.auth.params.AuthPNames;
import org.apache.http.client.CookieStore;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpHead;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.params.AuthPolicy;
import org.apache.http.client.params.HttpClientParams;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.impl.auth.NTLMSchemeFactory;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.DefaultHttpRequestRetryHandler;
import org.apache.http.impl.cookie.DateParseException;
import org.apache.http.impl.cookie.DateUtils;
import org.apache.http.message.BasicHeader;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.HttpContext;
import org.netkernel.client.http.representation.HttpCacheRepresentation;
import org.netkernel.client.http.representation.HttpStateRepresentation;
import org.netkernel.client.http.representation.IHttpClientConfig;
import org.netkernel.client.http.representation.OAuthCredentials;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;


public class HttpClientAccessorImpl extends StandardAccessorImpl
{	
	private static HttpClientAccessorImpl sInstance;
	
	private static final String NETKERNEL_USER_AGENT="1060 NetKernel HTTP Client 2.0.0  http://www.1060research.com";
	private static final String HTTP_HEADER_CONTENT_LENGTH="Content-Length";
	
	private static final String HTTP_ACCESSOR_STATUS_CODE_METADATA="HTTP_ACCESSOR_STATUS_CODE_METADATA";
	private static final String HTTP_ACCESSOR_RESPONSE_HEADERS_METADATA="HTTP_ACCESSOR_RESPONSE_HEADERS_METADATA";
	
	private static final String HTTP_METHOD_GET="httpGet";
	private static final int HTTP_METHOD_GET_INT=1;
	private static final String HTTP_METHOD_PUT="httpPut";
	private static final int HTTP_METHOD_PUT_INT=2;
	private static final String HTTP_METHOD_POST="httpPost";
	private static final int HTTP_METHOD_POST_INT=3;
	private static final String HTTP_METHOD_HEAD="httpHead";
	private static final int HTTP_METHOD_HEAD_INT=4;
	private static final String HTTP_METHOD_DELETE="httpDelete";
	private static final int HTTP_METHOD_DELETE_INT=5;
	private static final String HTTP_METHOD_PATCH="httpPatch";
	private static final int HTTP_METHOD_PATCH_INT=6;
	
	private static final String CONTENT_TYPE_HEADER="Content-Type";
	private static final String EXPIRES_HEADER="Expires";
	
	//private static final String HTTP_CONFIG_URI="res:/etc/HTTPClientConfig.xml";
	private static final String HTTP_CONFIG_URI_DEFAULT="res:/etc/DefaultHTTPClientConfig.xml";
	
	private static final HashMap mModes=new HashMap();
	
	public static final SchemeRegistry mHTTPClientSchemeRegistry = new SchemeRegistry();
	
	{
		mModes.put(HTTP_METHOD_GET, new Integer(HTTP_METHOD_GET_INT));
		mModes.put(HTTP_METHOD_PUT, new Integer(HTTP_METHOD_PUT_INT));		
		mModes.put(HTTP_METHOD_POST, new Integer(HTTP_METHOD_POST_INT));
		mModes.put(HTTP_METHOD_HEAD, new Integer(HTTP_METHOD_HEAD_INT));
		mModes.put(HTTP_METHOD_DELETE, new Integer(HTTP_METHOD_DELETE_INT));
		mModes.put(HTTP_METHOD_PATCH, new Integer(HTTP_METHOD_PATCH_INT));
	}
	
	/** Creates a new instance of HTTPClientAccessor */
	public HttpClientAccessorImpl()
	{	declareThreadSafe();
		System.getProperties().setProperty("httpclient.useragent", NETKERNEL_USER_AGENT);
		Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());

		sInstance=this;
	}
	
	public static HttpClientAccessorImpl getInstance()
	{	return sInstance;
	}
	
	@Override
	public void onDecommissionEndpoint() throws Exception
	{	
	}

	@Override
	public void onSource(INKFRequestContext aContext) throws Exception
	{	Object result=null;
		HttpContext state=new BasicHttpContext();
		Object stateref=null;
		Object cookieref=null;
		int statusCode=-1;
		HttpResponse response=null;
		IHDSNode responseHeaders=null;
		long expires=0;
		String mimetype=null;
		boolean cache=aContext.getThisRequest().argumentExists("cache");
		
		//Get the configuration - first by push, then by pull
		IHttpClientConfig httpConfig;
		if(aContext.getThisRequest().argumentExists("config"))
		{	httpConfig=aContext.source("arg:config", IHttpClientConfig.class);
		}
		else
		{	httpConfig=aContext.source(HTTP_CONFIG_URI_DEFAULT, IHttpClientConfig.class);
		}
		
		
		String url;
		if(aContext.getThisRequest().argumentExists("simple"))
		{	//Set the url from simple format
			url=aContext.getThisRequest().getArgumentValue("simple");
			HttpGet get=new HttpGet(url);
			try
			{	HttpClient client=getHttpClient(httpConfig, state, false, aContext, url);
	          	response=client.execute(get, state);
	          	statusCode=response.getStatusLine().getStatusCode();
	          	if(statusCode>=400)
	          	{	throw new Exception("HTTP Request Error :"+response.getStatusLine().getStatusCode());
	          	}
	          	try
	          	{
	          		result=getResponse(response);
	          	}
	          	catch(Exception e)
	          	{	get.abort();
	          		throw e;
	          	}
	          	responseHeaders=getResponseHeaders(response.getAllHeaders());
				mimetype=getContentType(response.getFirstHeader(CONTENT_TYPE_HEADER));
				expires=getExpires(response.getFirstHeader(EXPIRES_HEADER));
			}
			finally
			{	get.abort();
			}
			INKFResponse resp= aContext.createResponseFrom(result);
			resp.setHeader(HTTP_ACCESSOR_STATUS_CODE_METADATA, statusCode);
			resp.setHeader(HTTP_ACCESSOR_RESPONSE_HEADERS_METADATA,responseHeaders);
			resp.setMimeType(mimetype);
			if(0 < expires)
			{	resp.setExpiry(INKFResponse.EXPIRY_CONSTANT, expires);	//Expire at an absolute time
			}
			else
			{	resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
			}
		}
		else
		{	//Set the url
			url=aContext.getThisRequest().getArgumentValue("url");
			
			//Get state if there is any
			String stateuri=aContext.getThisRequest().getArgumentValue("state");
			if(stateuri!=null)
			{	if(!stateuri.startsWith(HttpStateAccessor.ACTIVE_HTTP_STATE))
				{ stateuri=aContext.source("arg:state",String.class);
				}
				HttpStateRepresentation sa=aContext.source(stateuri, HttpStateRepresentation.class);
				state=sa.getState();
			}
			stateref=state.getAttribute(ClientContext.COOKIE_STORE);
			if(stateref!=null)
			{	cookieref=((CookieStore)stateref).getCookies();				
			}
			
			//Oauth credentials?
			OAuthCredentials oauth=null;
			if(aContext.getThisRequest().argumentExists("oauth"))
			{	oauth=aContext.source("arg:oauth", OAuthCredentials.class);				
			}
			
			//User Specified Headers
			String headeruri=aContext.getThisRequest().getArgumentValue("headers");
			IHDSNode headers=null;
			if(headeruri!=null)
			{	headers=aContext.source("arg:headers", IHDSNode.class);
			}
					
			Integer mode=(Integer)mModes.get(aContext.getThisRequest().getArgumentValue("activeType"));
			switch (mode.intValue())
			{	case HTTP_METHOD_GET_INT:
					boolean remote=true;
					boolean shouldCache=false;
					HttpCacheRepresentation cacheRep=null;
					if(cache)
					{	//Source from cache
						INKFRequest req=aContext.createRequest(aContext.getThisRequest().getArgumentValue("cache"));
						req.addArgument("url", url);
						req.setRepresentationClass(HttpCacheRepresentation.class);
						cacheRep=(HttpCacheRepresentation)aContext.issueRequest(req);
						/*
						 * Should be test for expired
						 * 
						if(cacheRep!=null)
						{	remote=false;
							statusCode=HttpStatus.SC_OK;
							mimetype=cacheRep.getMimeType();
							result=cacheRep;
						}
						*/
					}
					if(remote)
					{	HttpGet get=new HttpGet(url);
						if(headers!=null)
						{	this.setRequestHeaders(get, headers);
						}
						if(cacheRep!=null)
						{	get.setHeader("If-None-Match", cacheRep.getTag());
						}
						try
						{	HttpClient client=getHttpClient(httpConfig, state, false, aContext, url);
							if(oauth!=null)
							{	oauthSignRequest(get, oauth);								
							}
		                  	response=client.execute(get, state);
		                  	statusCode=response.getStatusLine().getStatusCode();
		                  	if(statusCode==HttpStatus.SC_NOT_MODIFIED)
		                  	{	result=cacheRep;
		                  	}
		                  	else
		                  	{	
		                  		try
		                  		{
		                  			result=getResponse(response);
		                  		}
			                  	catch(Exception e)
			                  	{	get.abort();
			                  		throw e;
			                  	}
		                  		shouldCache=true;
		                  	}
		                  	responseHeaders=getResponseHeaders(response.getAllHeaders());
							mimetype=getContentType(response.getFirstHeader(CONTENT_TYPE_HEADER));
							expires=getExpires(response.getFirstHeader(EXPIRES_HEADER));
						}
						finally
						{	get.abort();
						}
						//Sink to cache
						if(cache && shouldCache)
						{	INKFRequest req=aContext.createRequest(aContext.getThisRequest().getArgumentValue("cache"));
							req.addArgument("url", url);
							String etag=(String)responseHeaders.getFirstValue("ETag");
							req.addPrimaryArgument(new HttpCacheRepresentation(etag, mimetype, 0, (ByteArrayRepresentation)result ));
							req.setVerb(INKFRequestReadOnly.VERB_SINK);
							aContext.issueRequest(req);
						}
					}
				break;
				
				case HTTP_METHOD_POST_INT:
					HttpPost post=new HttpPost(url);
					post.getParams().setParameter("http.protocol.expect-continue", Boolean.TRUE);
					if(headers!=null)
					{	this.setRequestHeaders(post, headers);
					}
					if(aContext.getThisRequest().argumentExists("body")) 
					{	//Currently we're only supporting single body argument
						INKFResponseReadOnly<IReadableBinaryStreamRepresentation> resp=aContext.sourceForResponse("arg:body", IReadableBinaryStreamRepresentation.class);
						IReadableBinaryStreamRepresentation rep=resp.getRepresentation();
						post.setEntity(new ReadableBinaryRequestEntity(rep, resp.getMimeType()));
					}
					else if(aContext.getThisRequest().argumentExists("nvp"))
					{	IHDSNode nvpa=(IHDSNode)aContext.source("arg:nvp",IHDSNode.class);
						List<NameValuePair> formparams = new ArrayList<NameValuePair>();
						for(IHDSNode n: nvpa.getChildren())
						{	formparams.add(new BasicNameValuePair(n.getName(), (String)n.getValue()));
						}
						UrlEncodedFormEntity entity = new UrlEncodedFormEntity(formparams, "UTF-8");
						post.setEntity(entity);
					}
					try
					{	HttpClient client=getHttpClient(httpConfig,state, true, aContext, url);
						if(oauth!=null)
						{	oauthSignRequest(post, oauth);								
						}
	                  	response=client.execute(post, state);
	                  	statusCode=response.getStatusLine().getStatusCode();
	                  	try
	                  	{
	                  		result=getResponse(response);
	                  	}
	                  	catch(Exception e)
	                  	{	post.abort();
	                  		throw e;
	                  	}
						responseHeaders=getResponseHeaders(response.getAllHeaders());
						mimetype=getContentType(response.getFirstHeader(CONTENT_TYPE_HEADER));
						expires=getExpires(response.getFirstHeader(EXPIRES_HEADER));
					}
					finally
					{	post.abort();
					}
				break;
				
				case HTTP_METHOD_PUT_INT:
				{
					HttpPut put=new HttpPut(url);
					put.getParams().setParameter("http.protocol.expect-continue", Boolean.TRUE);
					if(headers!=null)
					{	this.setRequestHeaders(put, headers);
					}
					INKFResponseReadOnly resp=aContext.sourceForResponse("arg:body", IReadableBinaryStreamRepresentation.class);
					IReadableBinaryStreamRepresentation rep=(IReadableBinaryStreamRepresentation)resp.getRepresentation();
					put.setEntity(new ReadableBinaryRequestEntity(rep, resp.getMimeType()));
					try
					{	HttpClient client=getHttpClient(httpConfig, state, true, aContext, url);
						if(oauth!=null)
						{	oauthSignRequest(put, oauth);								
						}
	                 	response=client.execute(put, state);
	                 	statusCode=response.getStatusLine().getStatusCode();
	                 	try
	                 	{
	                 		result=getResponse(response);
	                 	}
						catch(Exception e)
	                  	{	put.abort();
	                  		throw e;
	                  	}
						responseHeaders=getResponseHeaders(put.getAllHeaders());
						mimetype=getContentType(response.getFirstHeader(CONTENT_TYPE_HEADER));
						expires=getExpires(response.getFirstHeader(EXPIRES_HEADER));
					}
					finally
					{	put.abort();						
					}
				}
				break;
				case HTTP_METHOD_PATCH_INT:
				{
					HttpPatch patch=new HttpPatch(url);
					patch.getParams().setParameter("http.protocol.expect-continue", Boolean.TRUE);
					if(headers!=null)
					{	this.setRequestHeaders(patch, headers);
					}
					INKFResponseReadOnly resp=aContext.sourceForResponse("arg:body", IReadableBinaryStreamRepresentation.class);
					IReadableBinaryStreamRepresentation rep=(IReadableBinaryStreamRepresentation)resp.getRepresentation();
					patch.setEntity(new ReadableBinaryRequestEntity(rep, resp.getMimeType()));
					try
					{	HttpClient client=getHttpClient(httpConfig, state, true, aContext, url);
						if(oauth!=null)
						{	oauthSignRequest(patch, oauth);								
						}
	                 	response=client.execute(patch, state);
	                 	statusCode=response.getStatusLine().getStatusCode();
	                 	try
	                 	{
	                 		result=getResponse(response);
						}
	                  	catch(Exception e)
	                  	{	patch.abort();
	                  		throw e;
	                  	}
						responseHeaders=getResponseHeaders(response.getAllHeaders());
						mimetype=getContentType(response.getFirstHeader(CONTENT_TYPE_HEADER));
						expires=getExpires(response.getFirstHeader(EXPIRES_HEADER));
					}
					finally
					{	patch.abort();						
					}
				}
				break;
				
				case HTTP_METHOD_HEAD_INT:
					HttpHead head=new HttpHead(url);
					if(headers!=null)
					{	this.setRequestHeaders(head, headers);
					}
					try
					{	HttpClient client=getHttpClient(httpConfig, state, false, aContext, url);
						if(oauth!=null)
						{	oauthSignRequest(head, oauth);								
						}
	                  	response=client.execute(head, state);
	                  	statusCode=response.getStatusLine().getStatusCode();
	                  	responseHeaders=getResponseHeaders(response.getAllHeaders());
	                  	result=responseHeaders;
						mimetype=getContentType(response.getFirstHeader(CONTENT_TYPE_HEADER));
						expires=getExpires(response.getFirstHeader(EXPIRES_HEADER));
					}
					finally
					{	head.abort();						
					}
				break;
				case HTTP_METHOD_DELETE_INT:
					HttpDelete delete=new HttpDelete(url);
					if(headers!=null)
					{	this.setRequestHeaders(delete, headers);
					}
					try
					{	HttpClient client=getHttpClient(httpConfig, state, false, aContext, url);
						if(oauth!=null)
						{	oauthSignRequest(delete, oauth);								
						}
	                  	response=client.execute(delete, state);
	                  	statusCode=response.getStatusLine().getStatusCode();
	                  	try
	                  	{	result=getResponse(response);
	                  	}
	                  	catch(Exception e)
	                  	{	delete.abort();
	                  		throw e;
	                  	}
	                  	responseHeaders=getResponseHeaders(response.getAllHeaders());
						mimetype=getContentType(response.getFirstHeader(CONTENT_TYPE_HEADER));
						expires=getExpires(response.getFirstHeader(EXPIRES_HEADER));
					}
					finally
					{	delete.abort();						
					}
					
				break;
			}
			
			//Consume content so that connection is released to manager.
			if(response!=null)
			{	HttpEntity entity=response.getEntity();
				if(entity!=null)
				{	entity.consumeContent();
				}
			}
			
			//Sink updated state
			CookieStore cs=(CookieStore)state.getAttribute(ClientContext.COOKIE_STORE);
			if(cs!=null && !cookieref.equals(cs.getCookies()))				
			{	HttpStateRepresentation sa=new HttpStateRepresentation(state);
				aContext.sink(stateuri, sa);
			}
			
			INKFResponse resp= aContext.createResponseFrom(result);
			resp.setHeader(HTTP_ACCESSOR_STATUS_CODE_METADATA, statusCode);
			resp.setHeader(HTTP_ACCESSOR_RESPONSE_HEADERS_METADATA,responseHeaders);
			resp.setMimeType(mimetype);
			//resp.setNoCache();
			if(expires>0)
			{	resp.setExpiry(INKFResponse.EXPIRY_CONSTANT, expires);	//Expire at an absolute time
			}
			else
			{	resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
			}

		}
	}
	
	private void oauthSignRequest(HttpRequest request, OAuthCredentials credentials) throws Exception
	{	OAuthConsumer consumer=new CommonsHttpOAuthConsumer(credentials.getConsumerKey(), credentials.getConsumerSecret());
		consumer.setTokenWithSecret(credentials.getAccessToken(), credentials.getAccessTokenSecret());
		consumer.sign(request);
	}
	
	
	private IHDSNode getResponseHeaders(Header[] aHeaders)
	{	HDSBuilder b=new HDSBuilder();
		for(Header h: aHeaders)
		{	b.addNode(h.getName(),h.getValue());
		}
		return b.getRoot();
	}
		
	private void setRequestHeaders(HttpRequest method, IHDSNode headers)
	{	for(IHDSNode n: headers.getChildren())
		{	String value=(String)n.getValue();
			method.setHeader(n.getName(),value);
		}
	}
	
	private ByteArrayRepresentation getResponse(HttpResponse response) throws Exception
	{	return new ByteArrayRepresentation(getResponseInner(response).toByteArray());
	}
	
	private ByteArrayOutputStream getResponseInner(HttpResponse response) throws Exception
	{	long size;
		HttpEntity e=response.getEntity();
		InputStream bytes;
		if(e!=null)
		{
			size=e.getContentLength();
			if(size<=0)
			{	size=2048;			
			}
			bytes=e.getContent();
		}
		else
		{	byte[]empty=new byte[0];
			bytes=new ByteArrayInputStream(empty);
			size=1;
		}
		try
		{
			ByteArrayOutputStream baos=new ByteArrayOutputStream((int)size);
			org.netkernel.layer0.util.Utils.pipe(
				bytes,
				new BufferedOutputStream(baos)
			);
			return baos;
		}
		catch(IOException ex)
		{	throw ex;
		}
		catch(RuntimeException re)
		{	bytes.close();
			throw re;
		}
	}

	private String getContentType(Header h)
	{	String result;
		if(h!=null)
		{	String val=h.getValue();
			int i=val.indexOf(';');
			if(i>0)	result=val.substring(0,i);
			else result=val;
		}
		else result="content/unknown";
		return result;
	}
	
	private long getExpires(Header h)
	{	long result=0;
		try
		{	if(h!=null)
			{	Date d2=DateUtils.parseDate( h.getValue());
				result=d2.getTime();
			}
		}
		catch(DateParseException e){/*Must be expired*/}
		return result;
	}
	
	private HttpClient getHttpClient(IHttpClientConfig cfg, HttpContext state, boolean entityEnclosingMethod, INKFRequestContext aContext, String aURL) throws Exception
	{	HttpParams params = new BasicHttpParams();
		if(!entityEnclosingMethod)
		{	HttpClientParams.setRedirecting(params, cfg.followRedirects());
		}
		else
		{	//Entity enclosing methods cannot follow redirects without user intervention
			HttpClientParams.setRedirecting(params, false);
		}
		HttpConnectionParams.setConnectionTimeout(params, cfg.getConnectTimeout());
		HttpConnectionParams.setSoTimeout(params, cfg.getTimeout());
		
		if(!aContext.getThisRequest().argumentExists("state"))
		{	//Turn off cookie processing as we have no persistent state
			//Wish I knew how to do this with Apache 4 client !!
		}
		
		//Source Connection Manager
		ClientConnectionManager cm=null;
		if(aContext.getThisRequest().argumentExists("connectionManager"))
		{	cm=aContext.source("arg:connectionManager", ClientConnectionManager.class);			
		}
		else
		{	cm=aContext.source("active:httpClientDefaultConnectionManager", ClientConnectionManager.class);
		}
		
		//Instantiate Client
		DefaultHttpClient client=new DefaultHttpClient(cm, params);
		
		//Retry handler
		DefaultHttpRequestRetryHandler retryhandler=new DefaultHttpRequestRetryHandler(cfg.getNumberOfRetryAttempts(), false);
		client.setHttpRequestRetryHandler(retryhandler);
		
		String proxyHost=cfg.getProxyHost();
		if(proxyHost!=null)
		{	URI url=new URI(aURL);
			String host=url.getHost();
			if(!cfg.isProxyingExcluded(host))
			{
				HttpHost proxy = new HttpHost(proxyHost, cfg.getProxyPort());
				client.getParams().setParameter(ConnRoutePNames.DEFAULT_PROXY, proxy);
			
				String user=cfg.getProxyUser();
				String password=cfg.getProxyPassword();
				Credentials cred=null;
				if (cfg.getProxyNTLMDomain()!=null)
				{	cred = new NTCredentials(user, password, cfg.getProxyNTLMHost(), cfg.getProxyNTLMDomain());
					//client.getAuthSchemes().register("ntlm", new NTLMSchemeFactory());  - legacy used with the JCIFS library
					List<String> authpref = new ArrayList<String>();
					authpref.add(AuthPolicy.NTLM);
					client.getParams().setParameter(AuthPNames.PROXY_AUTH_PREF, authpref);
					aContext.logRaw(INKFRequestContext.LEVEL_DEBUG, "Registered NTLM proxy route and set PROXY_AUTH_PREF to NTLM" );
				}
				else if (user!=null && password!=null)
				{	cred=new UsernamePasswordCredentials(user,password);
				}
				if(cred!=null)
				{	CredentialsProvider cp=(CredentialsProvider)state.getAttribute(ClientContext.CREDS_PROVIDER);
					if(cp==null)
					{	cp=new BasicCredentialsProvider();
						state.setAttribute(ClientContext.CREDS_PROVIDER, cp);
					}
					AuthScope scope=new AuthScope(proxyHost, cfg.getProxyPort());
					cp.setCredentials(scope, cred);
				}
			}
		}
		
		//Deal with Client specific settings
		client.getParams().setParameter(CoreProtocolPNames.USE_EXPECT_CONTINUE, cfg.expectContinue());
		client.getParams().setParameter(CoreProtocolPNames.USER_AGENT, cfg.getUserAgent());
		client.getParams().setParameter(AuthPNames.TARGET_AUTH_PREF, cfg.getAuthPreferenceOrder());
		
		return client;
	}
	
	private class ReadableBinaryRequestEntity implements HttpEntity
	{	
		private IReadableBinaryStreamRepresentation mRBS;
		
		public void consumeContent() throws IOException
		{
		}

		public InputStream getContent() throws IOException, IllegalStateException
		{	return mRBS.getInputStream();
		}

		public Header getContentEncoding()
		{	return new BasicHeader("Content-Encoding", mRBS.getEncoding());
		}

		public boolean isChunked()
		{	return false;
		}

		public boolean isStreaming()
		{	return true;
		}

		public void writeTo(OutputStream out) throws IOException
		{	mRBS.write(out);
		}

		private String mType;
		
		public ReadableBinaryRequestEntity(IReadableBinaryStreamRepresentation aRBS, String aType)
		{	mRBS=aRBS;
			mType=aType;
		}

		public long getContentLength()
		{	return (long)mRBS.getContentLength();
		}

		public Header getContentType()
		{	return new BasicHeader(HttpClientAccessorImpl.CONTENT_TYPE_HEADER, mType);
		}

		public boolean isRepeatable()
		{	return true;
		}
	}
}
