package org.netkernel.client.http.endpoint;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.OAuthProvider;
import oauth.signpost.commonshttp.CommonsHttpOAuthConsumer;
import oauth.signpost.commonshttp.CommonsHttpOAuthProvider;
import oauth.signpost.http.HttpParameters;

import org.netkernel.client.http.representation.OAuthInitSettings;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class OAuthObtainAccessToken extends StandardAccessorImpl
{

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	
		OAuthInitSettings settings=context.source("arg:settings", OAuthInitSettings.class);
		
		// create a new service provider object and configure it with
        // the URLs which provide request tokens, access tokens, and
        // the URL to which users are sent in order to grant permission
        // to your application to access protected resources
        OAuthProvider provider = new CommonsHttpOAuthProvider(
        		settings.RequestTokenEndpointURL,
        		settings.AccessTokenEndpointURL,
        		settings.AuthorizeWebSiteURL
        );
        
        OAuthConsumer consumer = new CommonsHttpOAuthConsumer(
        		settings.ConsumerKey,
        		settings.ConsumerSecret
        );
		
        provider.setOAuth10a(true);
        //Re-establish preparation state to conclude access token download
        IHDSNode state=context.source("arg:prepareState", IHDSNode.class);
        String accessToken=(String)state.getFirstValue("/hds/accessToken");
        String accessTokenSecret=(String)state.getFirstValue("/hds/accessTokenSecret");
        consumer.setTokenWithSecret(accessToken,accessTokenSecret);
		
        HttpParameters params=new HttpParameters();
        IHDSNodeList nl=state.getNodes("/hds/*");
        for(IHDSNode n : nl)
        {	String name=n.getName();
        	if(name.startsWith("oauth"))
        	{	String value=(String)n.getValue();
        		params.put(name, value);
        		//System.out.println(name+"  : "+value);
        	}
        }
		consumer.setAdditionalParameters(params);
		
		String validationCode=context.source("arg:validationCode", String.class);
		
		provider.retrieveAccessToken(consumer, validationCode);
		
		HDSBuilder b=new HDSBuilder();
		b.pushNode("oauth");
		b.addNode("consumerKey", settings.ConsumerKey);
		b.addNode("consumerSecret", settings.ConsumerSecret);
		b.addNode("accessToken", consumer.getToken());
		b.addNode("accessTokenSecret", consumer.getTokenSecret());
		INKFResponse resp=context.createResponseFrom(b.getRoot());
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}

}
