package org.netkernel.client.http.endpoint;

import oauth.signpost.OAuth;
import oauth.signpost.OAuthConsumer;
import oauth.signpost.OAuthProvider;
import oauth.signpost.commonshttp.CommonsHttpOAuthConsumer;
import oauth.signpost.commonshttp.CommonsHttpOAuthProvider;
import oauth.signpost.http.HttpParameters;

import org.netkernel.client.http.representation.OAuthInitSettings;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class OAuthObtainRequestToken extends StandardAccessorImpl
{

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{
		OAuthInitSettings settings=context.source("arg:settings", OAuthInitSettings.class);
		
		// create a new service provider object and configure it with
        // the URLs which provide request tokens, access tokens, and
        // the URL to which users are sent in order to grant permission
        // to your application to access protected resources
        OAuthProvider provider = new CommonsHttpOAuthProvider(
        		settings.RequestTokenEndpointURL,
        		settings.AccessTokenEndpointURL,
        		settings.AuthorizeWebSiteURL
        );
        
        OAuthConsumer consumer = new CommonsHttpOAuthConsumer(
        		settings.ConsumerKey,
        		settings.ConsumerSecret
        );

        // fetches a request token from the service provider and builds
        // a url based on AUTHORIZE_WEBSITE_URL and CALLBACK_URL to
        // which your app must now send the user
        provider.setOAuth10a(true);
        String callback=settings.CallbackURL!=null?settings.CallbackURL:OAuth.OUT_OF_BAND;
        String authUrl = provider.retrieveRequestToken(consumer, callback);
        HDSBuilder b=new HDSBuilder();
        b.pushNode("hds");
        b.addNode("authorizeURL", authUrl);
		b.addNode("accessToken", consumer.getToken());
		b.addNode("accessTokenSecret", consumer.getTokenSecret());
		HttpParameters params=consumer.getRequestParameters();
		for(String key : params.keySet())
		{	b.addNode(key, params.getFirst(key));	
		}
        
		INKFResponse resp=context.createResponseFrom(b.getRoot());
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}

}
